/*
 * Copyright © 2021-2022 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of gaming-core.
 *
 * Gaming-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Gaming-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * gaming-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.core.domain;

import java.util.Objects;
import java.util.Optional;

/**
 * A foundation block for implementing players.
 *
 * @param <P> The type of allowed players.
 */
public abstract class AbstractPlayer<P extends Player<P>> implements Player<P> {

    /**
     * The name of this player.
     */
    private final String name;
    /**
     * The state of the player.
     */
    private PlayerState state;
    /**
     * The outcome of the player.
     */
    private Optional<Double> outcome;

    /**
     * Creates a player.
     *
     * @param name The name of the player.
     */
    protected AbstractPlayer(final String name) {
        this.name = Objects.requireNonNull(name, "name");
        this.state = PlayerState.PLAYING;
        this.outcome = Optional.empty();
    }

    /**
     * Creates a Demo player.
     *
     * @param source The {@link Player} to copy.
     */
    protected AbstractPlayer(final P source) {
        this.name = source.getName();
        this.state = source.getState();
        this.outcome = source.getOutcome();
    }

    @Override
    public boolean equals(final Object obj) {
        if (obj instanceof AbstractPlayer) {
            final AbstractPlayer<?> other = (AbstractPlayer<?>) obj;
            return this.name.equals(other.name) && this.state.equals(other.state)
                    && this.outcome.equals(other.outcome);
        }
        return false;
    }

    @Override
    public int hashCode() {
        return this.name.hashCode();
    }

    @Override
    public final String getName() {
        return this.name;
    }

    @Override
    public final PlayerState getState() {
        return this.state;
    }

    @Override
    public final void setState(final PlayerState newState) {
        this.state = newState;
        if (newState.equals(PlayerState.PLAYING)) {
            this.outcome = Optional.empty();
        }
    }

    @Override
    public final Optional<Double> getOutcome() {
        return this.outcome.isPresent() ? this.outcome : mapStateToOutcome(this.state);
    }

    @Override
    public final void setOutcome(final double newOutcome) {
        if (this.state.equals(PlayerState.PLAYING)) {
            throw new IllegalStateException(String.format("Cannot set outcome for player %s.", this.name));
        }
        this.outcome = Optional.of(newOutcome);
    }

    /**
     * Maps passed player state to the player's outcome. The outcome is unavailable
     * if the state is {@link PlayerState#PLAYING}. Otherwise, the outcome is computed from the player
     * state as follows:
     * <ul>
     * <li>1.0 if the state is {@link PlayerState#WON}</li>
     * <li>0.0 if the state is {@link PlayerState#DRAW}</li>
     * <li>-1.0 if the state is {@link PlayerState#LOST} or
     * {@link PlayerState#RESIGNED}</li>
     * </ul>
     *
     * @param state The player state.
     */
    public static Optional<Double> mapStateToOutcome(final PlayerState state) {
        switch (state) {
        case WON:
            return Optional.of(1.0);
        case LOST:
        case RESIGNED:
            return Optional.of(-1.0);
        case DRAW:
            return Optional.of(0.0);
        case PLAYING:
        default:
            return Optional.empty();
        }
    }
}
