/*
 * Copyright © 2020-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of gaming-core.
 *
 * Gaming-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Gaming-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * gaming-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.core.domain;

import java.util.Optional;
import java.util.Set;

/**
 * Receives events about the game.
 */
public interface Observer {

    /**
     * Indicates that the game has been started.
     *
     * @param game  The game.
     * @param state A copy of the game state.
     * @throws InterruptedException if the game has been interrupted while processing this event.
     */
    void started(Game<?, ?, ?, ?> game, State<?, ?> state) throws InterruptedException;

    /**
     * Indicates that a new set of players being able to make a move has been computed.
     *
     * @param game    The game.
     * @param state   A copy of the game state.
     * @param players The set of players. May be empty if no players have been chosen.
     * @throws InterruptedException if the game has been interrupted while processing this event.
     */
    void nextPlayersComputed(Game<?, ?, ?, ?> game, State<?, ?> state, Set<? extends Player<?>> players)
            throws InterruptedException;

    /**
     * Indicates that an illegal player has been rejected by the game.
     *
     * @param game   The game.
     * @param state  A copy of the game state.
     * @param player The player.
     * @throws InterruptedException if the game has been interrupted while processing this event.
     */
    void illegalPlayerRejected(Game<?, ?, ?, ?> game, State<?, ?> state, Player<?> player) throws InterruptedException;

    /**
     * Indicates that a legal move has been applied to the game state.
     *
     * @param game   The game.
     * @param state  A copy of the game state.
     * @param player The player.
     * @param move   The move.
     * @throws InterruptedException if the game has been interrupted while processing this event.
     */
    void legalMoveApplied(Game<?, ?, ?, ?> game, State<?, ?> state, Player<?> player, Move<?, ?> move)
            throws InterruptedException;

    /**
     * Indicates that an illegal move has been rejected by the game.
     *
     * @param game   The game.
     * @param state  A copy of the game state.
     * @param player The player.
     * @param move   The move. Can be empty if no move at all has been selected by the strategy.
     * @param reason The reason why the move is illegal.
     * @throws InterruptedException if the game has been interrupted while processing this event.
     */
    void illegalMoveRejected(Game<?, ?, ?, ?> game, State<?, ?> state, Player<?> player, Optional<Move<?, ?>> move,
            String reason) throws InterruptedException;

    /**
     * Indicates that a move is overdue, i.e. it has exceeded its time window.
     *
     * @param game       The game.
     * @param state      A copy of the game state.
     * @param player     The player.
     * @param chosenMove The move that has been chosen. Can be empty if no move is available (e.g. because a player has
     *                   already won the game).
     * @throws InterruptedException if the game has been interrupted while processing this event.
     */
    void overdueMoveRejected(Game<?, ?, ?, ?> game, State<?, ?> state, Player<?> player,
            Optional<Move<?, ?>> chosenMove)
            throws InterruptedException;

    /**
     * Indicates that a player has resigned.
     *
     * @param game   The game.
     * @param state  A copy of the game state.
     * @param player The player that has resigned.
     * @throws InterruptedException if the game has been interrupted while processing this event.
     */
    void playerResigned(Game<?, ?, ?, ?> game, State<?, ?> state, Player<?> player) throws InterruptedException;

    /**
     * Indicates that a player has been overtaken by some other player.
     *
     * @param game             The game.
     * @param state            A copy of the game state.
     * @param overtakenPlayer  The overtaken player.
     * @param overtakingPlayer The overtaking player.
     * @throws InterruptedException if the game has been interrupted while processing this event.
     */
    void playerOvertaken(Game<?, ?, ?, ?> game, State<?, ?> state, Player<?> overtakenPlayer,
            Player<?> overtakingPlayer)
            throws InterruptedException;

    /**
     * Indicates that the game has been finished.
     *
     * @param game  The game.
     * @param state A copy of the game state.
     * @throws InterruptedException if the game has been interrupted while processing this event.
     */
    void finished(Game<?, ?, ?, ?> game, State<?, ?> state) throws InterruptedException;
}
