/*
 * Copyright © 2020-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of gaming-core.
 *
 * Gaming-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Gaming-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * gaming-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.core.ui.type;

import java.util.Optional;

import de.fhdw.gaming.core.ui.InputProviderException;
import de.fhdw.gaming.core.ui.type.validator.Validator;

/**
 * Represents the type of a field.
 *
 * @param <T> The type of value.
 */
public interface FieldType<T> {

    /**
     * Accepts a {@link FieldTypeVisitor}.
     *
     * @param visitor The visitor to accept.
     * @throws FieldTypeException Wraps an exception thrown while processing a {@link FieldType}.
     */
    void accept(FieldTypeVisitor visitor) throws FieldTypeException;

    /**
     * Returns the default value.
     */
    Optional<T> getDefaultValue();

    /**
     * Returns information about the field type.
     */
    Optional<String> getInfo();

    /**
     * Checks if a value may be requested at all. This can happen if e.g. the set of valid values is empty and no
     * predefined value has been provided. Called before a value is requested from the user.
     *
     * @param id The ID of the value.
     * @throws InputProviderException if no value can ever match the criteria of this validator.
     */
    default void preInput(final String id) throws InputProviderException {
        // do nothing by default
    }

    /**
     * Validates a value.
     *
     * @param id    The ID of the value.
     * @param value The string value to validate.
     * @return The validated and correctly typed value.
     * @throws InputProviderException if the value is invalid.
     */
    T postInput(String id, String value) throws InputProviderException;

    /**
     * Creates a {@link FieldType} the values of which are validated by passed validators.
     *
     * @param validators The validators to use.
     * @return A {@link FieldType} with the validators attached. If no validators are passed, {@code this} is returned.
     */
    @SuppressWarnings("unchecked")
    default FieldType<T> validateBy(final Validator<T>... validators) {
        FieldType<T> fieldType = this;
        for (final Validator<T> validator : validators) {
            fieldType = new FieldTypeWithValidator<>(fieldType, validator);
        }
        return fieldType;
    }
}
